require 'active_support/core_ext/class/attribute'

require 'integrated_data/entity/integrator'
require 'integrated_data/entity/lookup'

module IntegratedData
  module Entity

    class << self
      def extended base; super
        base.class_attribute :lookups, instance_accessor: false, instance_predicate: false
        base.lookups = Set.new
      end
    end

    def new(identifiers={})
      attributes  = integrated(identifiers).attributes
      super(attributes).tap do |entity|
        entity.class_eval do
          @identifiers = identifiers
          @attributes  = attributes
        end
      end
    end

    def lookup(attribute, id: , identifier: nil, source: , strategy: :call, **options, &extractor)
      self.lookups = lookups.dup.add Lookup.new attribute, id, identifier, source, strategy, extractor, options
    end

  private

    def integrated(identifiers)
      Integrator.new(identifiers, available_lookups(identifiers.keys))
    end

    def available_lookups(identifiers)
      lookups.select do |lookup|
        identifiers.include? lookup.id
      end
    end

  end
end
