require 'integrated_data/entity/lookup'

module IntegratedData
  module Entity
    Integrator = Struct.new(:identifiers, :lookups)
    class Integrator

      def initialize(*args)
        @cache = {}
        super
      end

      def attributes
        Hash[
          lookups.group_by(&:attribute).map do |attribute, lookups|
            [ attribute, find_attribute_for(lookups, attribute) ]
          end
        ]
      end

    private

      def perform_lookup(lookup)
        @cache[lookup.cache_keys] ||= lookup.perform(identifiers[lookup.id])
      end

      def data_for(lookup, attribute)
        if data = perform_lookup(lookup)
          lookup.extract data, attribute
        end
      end

      def find_attribute_for(lookups, attribute)
        lookups.find do |lookup|
          data_for(lookup, attribute).tap do |data|
            return data if data
          end
        end
      end

    end
  end
end
