module IntegratedData
  module Entity
    Lookup = Struct.new(:attribute, :id, :identifier, :source, :strategy, :extractor, :options)
    class Lookup

      class << self
        def cache_keys
          [:identifier, :source, :strategy, :extractor, :options]
        end
      end

      def cache_keys
        self.class.cache_keys.map do |key|
          send key
        end
      end

      def perform(value)
        source.build(options).public_send(strategy, id, prepare(value))
      end

      def extract(data, attribute)
        if extractor
          extractor.call data, attribute
        else
          default_extractor data, attribute
        end
      end

    private

      class Error < Exception; end

      def prepare(value)
        identifier ? identifier.parse(value) : value
      end

      def default_extractor(data, attribute)
        if data.respond_to? :extract
          data.extract attribute
        elsif data.respond_to? :to_h
          data.to_h[attribute]
        elsif data.respond_to? attribute
          data.public_send attribute
        else
          raise Error, "couldn't extract #{attribute} from #{data.class}"
        end
      end

    end
  end
end
